﻿using System;
using System.Collections.Generic;
using System.Dynamic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;

namespace AdventOfCode2023.Day5
{
    public class Day5_2 : DayBase
    {
        string input = string.Empty;

        Regex rxSeeds = new Regex(@"(?<=seeds:[\s\d]+)\d+");
        Regex rxSeedToSoil = new Regex(@"(?<=seed-to-soil map:\r\n(\d+\s+\d+\s+\d+\r\n)*)(?<destinationRangeStart>\d+)\s+(?<sourceRangeStart>\d+)\s+(?<rangeLength>\d+)");
        Regex rxSoilToFertilizer = new Regex(@"(?<=soil-to-fertilizer map:\r\n(\d+\s+\d+\s+\d+\r\n)*)(?<destinationRangeStart>\d+)\s+(?<sourceRangeStart>\d+)\s+(?<rangeLength>\d+)");
        Regex rxFertilizerToWater = new Regex(@"(?<=fertilizer-to-water map:\r\n(\d+\s+\d+\s+\d+\r\n)*)(?<destinationRangeStart>\d+)\s+(?<sourceRangeStart>\d+)\s+(?<rangeLength>\d+)");
        Regex rxWaterToLight = new Regex(@"(?<=water-to-light map:\r\n(\d+\s+\d+\s+\d+\r\n)*)(?<destinationRangeStart>\d+)\s+(?<sourceRangeStart>\d+)\s+(?<rangeLength>\d+)");
        Regex rxLightToTemperature = new Regex(@"(?<=light-to-temperature map:\r\n(\d+\s+\d+\s+\d+\r\n)*)(?<destinationRangeStart>\d+)\s+(?<sourceRangeStart>\d+)\s+(?<rangeLength>\d+)");
        Regex rxTemperatureToHumidity = new Regex(@"(?<=temperature-to-humidity map:\r\n(\d+\s+\d+\s+\d+\r\n)*)(?<destinationRangeStart>\d+)\s+(?<sourceRangeStart>\d+)\s+(?<rangeLength>\d+)");
        Regex rxHumidityToLocation = new Regex(@"(?<=humidity-to-location map:\r\n(\d+\s+\d+\s+\d+\r\n)*)(?<destinationRangeStart>\d+)\s+(?<sourceRangeStart>\d+)\s+(?<rangeLength>\d+)");

        private readonly object lowestLocationLock = new object();
        private ulong lowestLocation = 0;

        public Day5_2()
        {
            input = GetInput("Day5.txt");
        }

        public Day5_2(string testInput)
        {
            input = testInput;
        }

        public override string Execute()
        {
            List<ulong> entries = rxSeeds.Matches(input).Select(x => ulong.Parse(x.Value)).ToList();

            var seeds = Enumerable.Range(0, entries.Count)
                .GroupBy(x => x / 2)
                .Select(x => x.Select(y => entries[y]).ToArray())
                .SelectMany(x => CreateRange(x[0], x[1]))
                .Select(x => (x, new ulong()));

            List<Map> seedToSoilMaps = rxSeedToSoil.Matches(input).Select(x => new Map(ulong.Parse(x.Groups["destinationRangeStart"].Value), ulong.Parse(x.Groups["sourceRangeStart"].Value), ulong.Parse(x.Groups["rangeLength"].Value))).ToList();
            List<Map> soilToFertilizer = rxSoilToFertilizer.Matches(input).Select(x => new Map(ulong.Parse(x.Groups["destinationRangeStart"].Value), ulong.Parse(x.Groups["sourceRangeStart"].Value), ulong.Parse(x.Groups["rangeLength"].Value))).ToList();
            List<Map> fertilizerToWater = rxFertilizerToWater.Matches(input).Select(x => new Map(ulong.Parse(x.Groups["destinationRangeStart"].Value), ulong.Parse(x.Groups["sourceRangeStart"].Value), ulong.Parse(x.Groups["rangeLength"].Value))).ToList();
            List<Map> waterToLight = rxWaterToLight.Matches(input).Select(x => new Map(ulong.Parse(x.Groups["destinationRangeStart"].Value), ulong.Parse(x.Groups["sourceRangeStart"].Value), ulong.Parse(x.Groups["rangeLength"].Value))).ToList();
            List<Map> lightToTemperature = rxLightToTemperature.Matches(input).Select(x => new Map(ulong.Parse(x.Groups["destinationRangeStart"].Value), ulong.Parse(x.Groups["sourceRangeStart"].Value), ulong.Parse(x.Groups["rangeLength"].Value))).ToList();
            List<Map> temperatureToHumidity = rxTemperatureToHumidity.Matches(input).Select(x => new Map(ulong.Parse(x.Groups["destinationRangeStart"].Value), ulong.Parse(x.Groups["sourceRangeStart"].Value), ulong.Parse(x.Groups["rangeLength"].Value))).ToList();
            List<Map> humidityToLocation = rxHumidityToLocation.Matches(input).Select(x => new Map(ulong.Parse(x.Groups["destinationRangeStart"].Value), ulong.Parse(x.Groups["sourceRangeStart"].Value), ulong.Parse(x.Groups["rangeLength"].Value))).ToList();

            Parallel.ForEach(seeds, seed =>
            {
                var soilNumber = seedToSoilMaps.Find(y => y.IsInSourceRange(seed.x))?.Convert(seed.x) ?? seed.x;
                var fertilizerNumber = soilToFertilizer.Find(y => y.IsInSourceRange(soilNumber))?.Convert(soilNumber) ?? soilNumber;
                var waterNumber = fertilizerToWater.Find(y => y.IsInSourceRange(fertilizerNumber))?.Convert(fertilizerNumber) ?? fertilizerNumber;
                var lightNumber = waterToLight.Find(y => y.IsInSourceRange(waterNumber))?.Convert(waterNumber) ?? waterNumber;
                var temperatureNumber = lightToTemperature.Find(y => y.IsInSourceRange(lightNumber))?.Convert(lightNumber) ?? lightNumber;
                var humidityNumber = temperatureToHumidity.Find(y => y.IsInSourceRange(temperatureNumber))?.Convert(temperatureNumber) ?? temperatureNumber;
                var locationNumber = humidityToLocation.Find(y => y.IsInSourceRange(humidityNumber))?.Convert(humidityNumber) ?? humidityNumber;

                lock(lowestLocationLock)
                {
                    if (lowestLocation == 0 || locationNumber < lowestLocation)
                    {
                        lowestLocation = locationNumber;
                    }
                }
            });

            return lowestLocation.ToString();
        }

        internal List<Map> GetMapsFromMap(IReadOnlyList<Map> maps, Map currentMap)
        {
            var currentMapSourceRange = currentMap.GetSourceRange();
            return GetAssociatedMaps(maps, currentMapSourceRange.rangeStart, currentMapSourceRange.rangeEnd).ToList();
        }

        internal IEnumerable<Map> GetAssociatedMaps(IReadOnlyList<Map> maps, ulong rangeStart, ulong rangeEnd) => maps.Where(x => (x.DestinationRangeStart <= rangeStart && x.DestinationRangeEnd >= rangeStart) || (x.DestinationRangeStart >= rangeStart && x.DestinationRangeStart <= rangeEnd));

        public IEnumerable<ulong> CreateRange(ulong start, ulong count)
        {
            var limit = start + count;

            while (start < limit)
            {
                yield return start;
                start++;
            }
        }
    }
}