﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Security.Cryptography.X509Certificates;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;

namespace AdventOfCode2023.Day7
{
    public class Day7_2 : DayBase
    {
        string input = string.Empty;
        Regex rxHand = new Regex(@"^(?<hand>[\d\w]{5})\s*(?<bid>\d+)", RegexOptions.Multiline);

        public Day7_2()
        {
            input = GetInput("Day7.txt");
        }

        public Day7_2(string testInput)
        {
            input = testInput;
        }

        public override string Execute()
        {
            var hands = rxHand.Matches(input)
                              .Select(x => new Hand_Day2(x.Groups["hand"].Value, int.Parse(x.Groups["bid"].Value)))
                              .OrderBy(x => x.GetHandStrength())
                              .ThenBy(x => x.Cards[0])
                              .ThenBy(x => x.Cards[1])
                              .ThenBy(x => x.Cards[2])
                              .ThenBy(x => x.Cards[3])
                              .ThenBy(x => x.Cards[4]);
            return hands.Select((hand, index) => hand.Bid * (index + 1)).Sum().ToString();
        }


    }

    internal class Hand_Day2
    {
        private string _hand;
        private int[] _cards;
        private int _bid;

        public Hand_Day2(string hand, int bid)
        {
            _hand = hand;
            _cards = hand.Select(CardToInt).ToArray();
            _bid = bid;
        }

        public int GetHandStrength()
        {
            var cardGroups = _hand.GroupBy(x => x).OrderByDescending(x => x.Count());
            var jokers = cardGroups.FirstOrDefault(x => x.Key == 'J')?.Count() ?? 0;
            var noJokerCardGroups = cardGroups;
            if (jokers > 0)
            {
                noJokerCardGroups = cardGroups.Where(x => x.Key != 'J').OrderByDescending(x => x.Count());
            }
            return (noJokerCardGroups.FirstOrDefault()?.Count() + jokers ?? 0 + jokers)  switch
            {
                5 => 6,
                4 => 5,
                3 => noJokerCardGroups.ElementAtOrDefault(1)?.Count() == 2 ? 4 : 3,
                2 => noJokerCardGroups.ElementAtOrDefault(1)?.Count() == 2 ? 2 : 1,
                1 => 0,
                _ => throw new ArgumentException($"Too many or no cards found."),
            };
        }

        private static int CardToInt(char x)
        {
            return x switch
            {
                'A' => 12,
                'K' => 11,
                'Q' => 10,
                'T' => 9,
                '9' => 8,
                '8' => 7,
                '7' => 6,
                '6' => 5,
                '5' => 4,
                '4' => 3,
                '3' => 2,
                '2' => 1,
                'J' => 0,
                _ => throw new ArgumentException($"Can't convert {x} to int."),
            };
        }

        public int[] Cards { get => _cards; }
        public int Bid { get => _bid; }
    }
}
